/*
 * This file has been slightly modified by NRL for use with IPv6+IPsec.
 * Search for INET6 and/or IPSEC to see the blocks where this happened.
 * See the NRL Copyright notice for conditions on the modifications.
 */
/*
 * Copyright (c) 1990, 1991, 1993, 1994, 1995, 1996
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
static const char rcsid[] =
    "@(#) $Header: /home/cmetz/src.CVS/ipv6/tcpdump-3.3.1a2/print-ppp.c,v 1.1 1997/01/15 18:49:15 cmetz Exp $ (LBL)";
#endif

#include <sys/param.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/ioctl.h>

#if __STDC__
struct mbuf;
struct rtentry;
#endif
#include <net/if.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>

#include <ctype.h>
#include <netdb.h>
#include <pcap.h>
#include <signal.h>
#include <stdio.h>

#include "interface.h"
#include "addrtoname.h"

/*
 * XXX BSD/OS and FreeBSD (and probably others) do not use the same
 * "link layer" header that the original LBL CSLIP used. The following
 * hack at least allows PPP to work with a live source. However, it
 * totally breaks the ability to create a PPP savefile on one type of
 * system and display it on another. A future version of
 * libpcap/tcpdump will try to address this in a more reasonable
 * mannor.
 */
#if defined(bsdi)
#define PPP_HDRLEN 24
#elif defined(__FreeBSD__)
/* XXX untested and probably wrong */
#define PPP_HDRLEN 24
#else
/* XXX This goes somewhere else. */
#define PPP_HDRLEN 4
#endif

/* Forwards */
static void ppp_hdlc_print(const u_char *, int);

static void
ppp_hdlc_print(register const u_char * p, register int length)
{
	printf("%c %4d %02x %04x: ", p[0] ? 'O' : 'I', length,
	       p[1], ntohs(*(u_short *)&p[2]));
}

void
ppp_if_print(u_char *user, const struct pcap_pkthdr *h,
	     register const u_char *p)
{
	register u_int length = h->len;
	register u_int caplen = h->caplen;
	const struct ip *ip;

	ts_print(&h->ts);

	if (caplen < PPP_HDRLEN) {
		printf("[|ppp]");
		goto out;
	}

	/*
	 * Some printers want to get back at the link level addresses,
	 * and/or check that they're not walking off the end of the packet.
	 * Rather than pass them all the way down, we set these globals.
	 */
	packetp = p;
	snapend = p + caplen;

	if (eflag)
		ppp_hdlc_print(p, length);

	length -= PPP_HDRLEN;
	ip = (struct ip *)(p + PPP_HDRLEN);
#ifdef INET6
	if (ip->ip_v==4) 
	  ip_print((const u_char *)ip, length);
        else  
	  ipv6_print((const u_char *)ip, length);
#else
	ip_print((const u_char *)ip, length);
#endif /* INET6 */

	if (xflag)
		default_print((const u_char *)ip, caplen - PPP_HDRLEN);
	if (Aflag)
	        ascii_print((const u_char *)ip, caplen - PPP_HDRLEN);
out:
	putchar('\n');
}
